function [models, modelfor, modeldist] = NT_build_all_models(distmx, ...
                                               FILES,AUDIOPATH,AUDIOEXT, ...
                                               CLSTFACT, ...
                                               CLSTPCNTILE,BLOCK,...
                                               NFRAME,NTTRIES, ...
                                               NTLOWE, NCOMP, ...
                                               MERGE_KLTHRESH, VAR_SCALEUP, ...
                                               LABELPATH,LABELEXT,LABELCODE,...
                                               TXPROB,USEVX,DESQTH,...
                                               NTMINPROP,NTVOLTHRESH, ...
                                               VERBOSE, VIEWMODELS)
% models = NT_build_all_models(distmx, ...)
%     Take a set of GMMs describing individual files; cluster them
%     when similar, and return NT-identifying models for each cluster.
% 2012-05-01 Dan Ellis dpwe@ee.columbia.edu

% We have an upper limit on how different tracks can be and still
% count as cohort
%gdthresh = 0.25*percentile(distmx(:),0.8);
% Should be based on very similar tracks
%distpcntl = percentile(distmx(:), CLSTPCNTILE);
ddim = length(distmx);
if ddim == 1
  % distance "matrix" is 1 x 1
  distpcntl = distmx(1,1);
else
  distpcntl = percentile(distmx(find(1-eye(ddim))), CLSTPCNTILE);
end
%disp(['percentile ',num2str(CLSTPCNTILE),' of dists is ',num2str(distpcntl)]);
distthresh = CLSTFACT*distpcntl;

if ddim > 1 && VIEWMODELS
  imagesc(distmx); axis('xy'); colorbar
  outline(distmx,distthresh,'r');
  title(['distthresh = ',num2str(distthresh)]);
  pause
end

% Precompute cohorts for every seed (even though we won't use them all)
[dists,distix] = sort(distmx,2);
% mask of how much of each row gets kept
incohortmask = (dists <= distthresh);
  
% Build models based on cohorts until all files have a
% corresponding model

ncohort = 0;
done = zeros(1,length(FILES));
while length(find(done==0))>0

  % Choose next as the cluster with the most unseen files
  [mm,nexttodo] = max(sum(incohortmask.*(done(distix)==0), 2));
  cohort = distix(nexttodo,find(incohortmask(nexttodo,:)));
  
  ncohort = ncohort+1;
  if length(cohort) < BLOCK
    ddisp(['warn: only ',num2str(length(cohort)),' similar files found'...
           ' for ',FILES{nexttodo}], VERBOSE);
  end
  % run findNT multiple times, keep best (most compact) volume
  % (tightest cluster), or one with best agreement with existing labels
  ddisp([datestr(rem(now,1),'HH:MM:SS'),' cohort ',num2str(ncohort),': ', ...
         sprintf('%d(%d) ', [cohort; done(cohort)])], VERBOSE);
  bestmetric = 0;

  % Get all the features for this cohort
  COHFILES = FILES(cohort(1:min(length(cohort),BLOCK)));
  data = NT_get_data(addprefixsuffix(COHFILES, AUDIOPATH, AUDIOEXT), ...
                    round(NFRAME/length(COHFILES)), USEVX, DESQTH, VERBOSE);
  for i = 1:NTTRIES
    % Build the GMM model with a designated NT component
    [model, NT] = NT_buildmodel(data, VERBOSE, NCOMP, ...
                                MERGE_KLTHRESH, VAR_SCALEUP, ...
                                NTMINPROP, NTVOLTHRESH);

    if NTLOWE; NT = NT(1); else NT = NT(end); end

    % Figure distance between NT component and rest of model
    NTmodel.mean = model.mean(:,NT);
    NTmodel.weight = ones(length(NT),1);
    NTmodel.cov = model.cov(:,:,NT);
    NTmodel.diag = 0;
    nNTmodel = model;
    nNTmodel.weight(NT) = 0;
    %gmm_print(NTmodel);
    %gmm_print(nNTmodel);
    NTKLsep = gmm_distance(NTmodel, nNTmodel);
    
    
    % If we have labels, use agreement with labels to choose best
    % model.  Otherwise, go for minimum-volume mode
      
    % "volume" of chosen NT cluster
    foundvol = det(squeeze(model.cov(:,:,NT(1)))).^(1/size(data,2));

    modalscore = gmm_modal_score(model, data, NT);
    if length(LABELPATH) > 0
      % agreement with GT labels...
      [SNT,CNT] = NT_scoremodel(model,NT,...
                                COHFILES,AUDIOPATH,AUDIOEXT, ...
                                LABELPATH,LABELEXT,LABELCODE,...
                                TXPROB,USEVX,DESQTH);
      metric = SNT;
    else
      %metric = 1/foundvol;
      metric = modalscore;
    end
      
    isbest = '';
    if metric > bestmetric
      bestmetric = metric;
      bestmodel = model;
      bestmodel.NT = NT;
      bestmodel.KLsep = NTKLsep;
      bestmodel.modalscore = modalscore;
      isbest = 'best';
    end
    ddisp(sprintf('Try %d vol %.2f ctr %.2f,%.2f KLSEP %.1f modal %.1f metric %.2f %s', ...
                  i, foundvol, ...
                  model.mean(1,NT(1)), model.mean(2,NT(1)), ...
                  NTKLsep, modalscore, metric, ...
                  isbest), ...
          VERBOSE);
    if VIEWMODELS
      % debug
      PAUSE = (NTTRIES > 1);
      gmm_plotdata(data, model, NT(1), PAUSE);
      % debug
    end
  end
  bestmodel.name = FILES{nexttodo};
  bestmodel.cohort = COHFILES;
  models(ncohort) = bestmodel;
  done(cohort) = done(cohort) + 1;
  modelfor(cohort) = ncohort;
  modeldist(cohort) = dists(nexttodo,find(incohortmask(nexttodo,:)));
end

ddisp([datestr(rem(now,1),'HH:MM:SS'), ' ', num2str(ncohort), ...
       ' cohorts, done= ',sprintf('%d ',done), ...
       ' (',sprintf('%.1f',sum(done)/length(done)),'x)'], VERBOSE);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ddisp(msg, verb)
if verb > 0
  disp(msg);
end
% verb also carries file descriptor of log file
if verb ~= 0
  if verb ~= 1 && verb ~= 2
    fprintf(abs(verb), '%s\n', msg);
  end
end
