function [model,NT] = NT_buildmodel(mfcc,VERBOSE,NCOMP, ...
                                    MERGE_KLTHRESH,VAR_SCALEUP, ...
                                    NTMINPROP, NTVOLTHRESH)
% [model,NT] = NT_buildmodel(mfcc,VERBOSE,NCOMP,MERGE_KLTHRESH,VAR_SCALEUP,NTMINPROP,NTVOLTHRESH)
%    mfcc = matrix of data, each row is a vector.
%    Build a gaussian mixture model, where NT'th component is
%    identified as NT (most compact high-energy mode, then low-energy).
% 2012-02-27 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2; VERBOSE = 0; end
if nargin < 3; NCOMP = 16; end
% Do we add a broad gaussian underneath the identified NT mode?
if nargin < 4; MERGE_KLTHRESH = 10; end
% How much larger to make decoy than primary gaussian
if nargin <5; VAR_SCALEUP = 0.0; end
if nargin <6; NTMINPROP=0.025; end % was .05
if nargin <7; NTVOLTHRESH=2.0; end 

ADD_DECOY = (VAR_SCALEUP > 0);

% To be a candidate for the NT cluster, a final (merged) cluster
% must account for at least NTMINPROP of all the frames, and must
% have a volume within NTVOLTHRESH of the smallest
%NTMINPROP = 0.025; % was .05
%NTVOLTHRESH = 2.0;

if VERBOSE; disp('building model...'); end

ndims = size(mfcc,2);

% Make models
MERGE_SMALLTHRESH = 2.0;
%MERGE_MINGPROB = 0.01;
%MERGE_KLTHRESH = 15;
%MERGE_KLTHRESH = 10;
model = gmm_estmerge(mfcc, NCOMP, MERGE_SMALLTHRESH, MERGE_KLTHRESH, VERBOSE>1);

% OK, how much left?
nclust = size(model.mean,2);
% find "volume" of gaussians
vol = zeros(1,nclust);
for i = 1:nclust
  vol(i) = det(squeeze(model.cov(:,:,i))).^(1/ndims);
end

% designate NT as the gaussian with (a) at least 5% of frames, and
% (b) volume within factor of 2 of smallest, and (c) highest
% average log energy (C0)
%[vv,xx] = sort(vol+(max(vol)*(N < 0.05*sum(N))));
goodN = (model.weight > NTMINPROP*sum(model.weight));
goodV = vol < NTVOLTHRESH*min(vol);
% lowest energy cluster comes first
[vv,xx_low] = sort((max(model.mean(1,:))-model.mean(1,:)).*goodN.*goodV, ...
                   'descend');
% or highest
[vv,xx_hi] = sort((model.mean(1,:)-min(model.mean(1,:))).*goodN.*goodV, ...
                  'descend'); 
% So the NT component is...
NT = [xx_low(1),xx_hi(1)];
if NT(1) == NT(2)
  NT = NT(1);
end

if ADD_DECOY
  % Add a decoy as the same NT model with scaled-up variance
  for i = 1:length(NT)
    model = gmm_addcomponent(model, ...
                             model.mean(:,NT(i)), ...
                             VAR_SCALEUP*model.cov(:,:,NT(i)), 0);
  end
end
    
if VERBOSE 
  gmm_print(model,[],NT);
%  gmm_plot(model, [], 1.0, 'r', 0, NT);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function modelout = gmm_addcomponent(modelin, mean, cov, wt)
% Append a new component to a GMM model structure
if nargin < 4;  wt = 0;  end

modelout = modelin;
newclust = size(modelout.mean,2)+1;
modelout.mean(:,newclust) = mean;
modelout.cov(:,:,newclust) = cov;
modelout.weight(newclust) = wt;
