function Z = NT_desquelch_labs(L,DESM)
% Z = NT_desquelch_labs(L,DESM)
%    Remove "squelch" times described in DESM rows from the 
%    label intervals (3col) described in L.  Return 3 col.
% 2012-05-22 Dan Ellis dpwe@ee.columbia.edu

% Convert L to 2cols
if size(L,2) == 3
  Y = lab3col_to_lab2col(L);
else
  % L passed in as 2 col
  Y = L;
end

Ylen = size(Y,1);
% Convert DESM to 2cols
DSlen = size(DESM,1);
D = [reshape(DESM',2*DSlen,1), repmat([1;0],DSlen,1)];
Dlen = size(D,1);

% Stuff for output
Z = zeros(size(Y,1),2);
Zpos = 0;

Dpos = 1;
Ypos = 1;

% Combined, time-sorted list of L and DESM labels

% Algorithm
insquelch = 0;
losttime = 0;
% foreach time in sorted list
%   if label == squelch on
%     insquelch = 1;
%     lastsquelchontime = thistime;
%   elseif label == squelch off
%     insquelch = 0;
%     losttime = losttime + (thistime - lastsquelchontime);
%     (if last emitted label ~= mostrecentlabel)
%       next output = [thistime - losttime, mostrecentlabel];
%   elseif label == original label
%     if insquelch == 0
%       next output = [labeltime-losttime, label];
%     end
%     mostrecentlabel = thislabel;
%   end
% end
done = 0;
while (Dpos <= Dlen) || (Ypos <= Ylen)
  if Ypos <= Ylen
    nextYtime = Y(Ypos,1);
  else
    nextYtime = Inf;
  end
  if Dpos <= Dlen
    nextDtime = D(Dpos,1);
  else
    nextDtime = Inf;
  end
  if nextDtime < nextYtime
    % next label is squelch
    if D(Dpos,2) == 1
      % squelch on
      insquelch = 1;
      lastsquelchontime = nextDtime;
    else
      % squelch off
      insquelch = 0;
      losttime = losttime + (nextDtime - lastsquelchontime);
      Zpos = Zpos + 1;
      Z(Zpos,:) = [nextDtime - losttime, mostrecentlabel];
    end
    Dpos = Dpos + 1;
  else
    % next label is actual label
    mostrecentlabel = Y(Ypos,2);
    if ~insquelch
      % propagate label if we're not in squelch
      Zpos = Zpos + 1;
      Z(Zpos,:) = [nextYtime - losttime, mostrecentlabel];
    end
    Ypos = Ypos + 1;
  end
end

    
Z = lab2col_to_lab3col(Z(1:Zpos,:));
