function [mfcc,mlen,desm] = NT_get_data(FNLIST,NFRAME,USEVX,DESQTH,VERBOSE)
% [mfcc,mlen,desm] = NT_get_data(X,NFRAME,USEVX,DESQTH,VERBOSE)
%    Get MFCC data for:
%      X = string: load this soundfile, return its features
%      X = cell array of strings: load every soundfile, return
%      concatenated features.
%      mfcc is just one big matrix, one row per feature frame
%      mlen is the number of frames associated with each file,
%        e.g. for when X is a cell array.
%      desm is the desquelch map for the last file.
% 2012-04-27 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2;  NFRAME = 0; end
if nargin < 3;  USEVX = 1; end
if nargin < 4;  DESQTH = -70; end
if nargin < 5;  VERBOSE = 0; end

if isstr(FNLIST)
  % Make it a single-entry list
  X = FNLIST;
  FNLIST = cell(1);
  FNLIST{1} = X;
end

mfcc = [];
mlen = [];
for i = 1:length(FNLIST);
  [mfccpart,desm,hopt] = cache_results(@nt_mfcc_vx_desquelch, ...
                                       {FNLIST{i}, DESQTH, USEVX, VERBOSE>1}, ...
                                       '', '', VERBOSE);
%    % per-file energy (c0) normalization
%    mfccpart(:,1) = mfccpart(:,1) - percentile(mfccpart(:,1),0.99) ...
%        + 100;
% not needed now we have file clustering in findNTs.m
  % keep subsets of frames for quicker modeling
  if NFRAME > 0 && size(mfccpart,1) > NFRAME
    mfccpart = mfccpart(randi(size(mfccpart,1),1,NFRAME), :);
  end
  mfcc = [mfcc;mfccpart];
  mlen = [mlen,size(mfccpart,1)];
end

% Filter dimensions
if USEVX
  % just 4 dims, but include voicing as 2nd
  keepdims = [1,17,2:3];
else
  keepdims = [1:4];
  %keepdims = [1:8];
end

mfcc = mfcc(:,keepdims);
