function [S,C,nts] = NT_scoremodel(model,NT,FILES,AUDIOPATH,AUDIOEXT,LABELPATH,LABELEXT,LABELCODE,TXPROB,USEVX,DESQTH)
% [S,C,nts] = NT_scoremodel(model,NT,FILES,AUDIOPATH,AUDIOEXT,LABELPATH,LABELEXT,LABELCODE,TXPROB,USEVX,DESQTH)
%        Calculate the NT-to-NT agreement for <model> running over
%        <FILES> whose ground-truth is given by files in LABELPATH
% 2012-05-01 Dan Ellis dpwe@ee.columbia.edu

if isstr(FILES)
  FILES = {FILES};
end

nfiles = length(FILES);

IX = {'S','NS','NT','RX','RM'};
SPx = 1; NSx = 2; NTx = 3; RXx = 4; RMx = 5;
frametime = 0.010; % we assume

C = zeros(2,2);

for i = 1:nfiles

  %disp(['NT_sc: ', FILES{i}]);
  
  % Check against existing labels?
  LABFILE = fullfile(LABELPATH,[FILES{i},LABELEXT]);
  if exist(LABFILE,'file') > 0

    % Load the features
    [data,mlen,desm] = NT_get_data(fullfile(AUDIOPATH,[FILES{i},AUDIOEXT]), ...
                                   0,USEVX,DESQTH);

    % Decode 
    SLOWDECODE = 0;
    nts = NT_decode(data, model, NT, TXPROB, SLOWDECODE);
  
    [L,IX] = read_label_file(LABFILE, IX, LABELCODE, 'NT');
    L = NT_desquelch_labs(L,desm);
    NRIX = find(L(:,3)==NTx | L(:,3)==RXx | L(:,3)==RMx);
    NRL = L(NRIX,:);
    NRL(:,3) = 1;  % dummy
    if length(NRIX) > 0 && length(nts) > 0
      [SNT,CNT] = compare_labels(nts, NRL);
      C = C+CNT;
    end
  else
    disp(['warn: label file ',LABFILE,' not found']);
  end
end

NOCHORD = 0;
XX = C(NOCHORD+1, NOCHORD+1);

S = (sum(diag(C))-XX) / (sum(C(:))-XX + (sum(C(:))==0));
% (return zero if no label files found)

