function gmm_plotdata(data, model, NT, PAUSE)
% gmm_plotdata(data, model, NT, PAUSE)
%   Make a 2D histogram of the first 2 dims of data rows in <data>
%   then add ellipses corresponding to the components of GMM
%   <model> highlighting the components named in <NT>.  If <PAUSE>,
%   pause afterwards.
% 2012-05-21 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2; model = []; end
if nargin < 3; NT = []; end
if nargin < 4; PAUSE = 0; end

[ndata,ndim] = size(data);

% Visualize data and matching model
xx = percentile(data(:,1),[0.001 0.999]);
dxx = diff(xx);
xx = [(xx(1)-dxx/2):dxx/100:(xx(2)+dxx/2)];

if ndim > 1
  yy = percentile(data(:,2),[0.001 0.999]);
  dyy = diff(yy);
  yy = [(yy(1)-dyy/2):dyy/100:(yy(2)+dyy/2)];
  Y = hist2(data(:,1:2),xx,yy)';
  imagesc(xx,yy,log(1+Y)); axis('xy'); colormap(1-gray);
else
  [histn, histx] = hist(data(:,1),100);
  bar(histx, histn);
end
if length(model) > 0
  gmm_plot(model, [], 1.0, 'r', 0, NT);
end
if PAUSE
  pause;
end
