function [r,p,d] = gmm_val(model, x)
% [r,p,d] = gmmval(model, x)  Evaluate a gaussian mixture model at points
%    x is a set of data points, one per row.  model defines 
%    a Gaussian mixture model's means, covariances and mix priors. 
%    r returns the full GMM evaluated at each x; 
%    p returns the posterior of each point belonging to each mix 
%    component.
%    d returns the mahalanobis distances (reciprocal mean reciprocal).
% 2001-02-11 dpwe@ee.columbia.edu
% $Header: $

[ndim, nmix] = size(model.mean);
ndat = size(x,1);

r = zeros(ndat,1);

p = zeros(ndat, nmix);
d = zeros(ndat, 1);

pistuff = (2*pi) ^ -(ndim/2);

weight = model.weight;
if sum(weight) ~= 1
%  disp(['Warn: renormalizing model weights that sum to ', ...
%        num2str(sum(weight))]);
  weight = weight/sum(weight);
end

% Calculate posterior data memberships for each component
for k = 1:nmix
  % Reconstruct covar mx
  if nmix > 1
    cv = model.cov(:,:,k);
  else
    cv = model.cov;
  end
%  mu = ones(ndat,1)*m(k,:);
%  xmm = x - mu;
  xmm = x - repmat(model.mean(:,k)',ndat,1);
  % Evaluate Gaussians
  if ndim == 1
    % Matlab syntax bites us
    %px = pistuff*exp(-0.5*   (xmm'.*(inv(cv)*xmm')))/sqrt(det(cv));
    dx = (xmm'.*(inv(cv)*xmm'));
  else
    %px = pistuff*exp(-0.5*sum(xmm'.*(inv(cv)*xmm')))/sqrt(det(cv));
    dx = sum(xmm'.*(inv(cv)*xmm'));
  end
  px = pistuff*exp(-0.5*dx)/sqrt(det(cv));
  p(:,k) = weight(k)*px';
  % divinding dx by ndim makes the expected distance peak at 1.0
  % regardless of the number of dimensions
  % geometric mean i.e. sum[w(i)]/sum[w(i)/d(i)] over mix
  % components i
  d = d + weight(k)./sqrt(dx'/ndim);
end

r = sum(p,2);
% normalize weighted likelihoods into posteriors
p = p./repmat(r,1,nmix);
d = 1./d;

