function Yout = hist2(X,W,M,N)
% Y = hist2(X,M,N) or hist2(X,W,M,N)   Form a 2-D histogram
%    X consists of 2-D data rows.  Y is a matrix of counts of 
%    values of X falling into each bin, defined by the values in 
%    M and N i.e. size(Y) = [length(M),length(N)].
%    Optional W is a weighting for each point in X.
% 2004-04-27 dpwe@ee.columbia.edu

nx = size(X,1);

if nargin == 1
  nbins = round(nx/20);
elseif nargin == 2
  if length(W) == 1
    nbins = W;
    W = ones(nx,1);
  else
    nbins = round(nx/20);
  end
end


if nargin == 3
  % old, non-weighted version with ranges provided
  N = M;
  M = W;
end

if nargin == 1 || nargin == 2
  % just data points, guess ranges
  minx = min(X(:,1)); maxx = max(X(:,1));
  M = minx:((maxx-minx)/nbins):maxx;
  miny = min(X(:,2)); maxy = max(X(:,2));
  N = miny:((maxy-miny)/nbins):maxy;
end

if nargin == 1 || nargin == 3
  % old, non-weighted version
  W = ones(nx,1);
end

Y = zeros(length(M),length(N));

for i = 1:nx
  mm = abs(M - X(i,1));
  nn = abs(N - X(i,2));
  mm = find(mm == min(mm));
  nn = find(nn == min(nn));
  Y(mm,nn) = Y(mm,nn)+W(i);
end

if nargout > 0
  Yout = Y;
else
  imagesc(N,M,Y);
  axis('xy');
end
