function [Z,E] = stxcorr(X,Y,W,H,N)
% [Z,E] = stxcorr(X,Y,W,H,N)  short-time cross-correlation
%    X and Y are segmented into W-point stretches, hopped by H samples. 
%    The cross-correlation of these, out to +/- N points, are returned 
%    as columns of Z.
%    Result is timing of Y relative to X.
%    E returns theoretical maximum (harmonic mean energy) for each
%    col of Z.
% 2004-04-28 dpwe@ee.columbia.edu

if nargin < 3
  W = 2000;
end
if nargin < 4
  H = round(W/2);
end
if nargin < 5
  N = W;
end

npts = 2*N+1;
nfr = 1+floor((min(length(X),length(Y))-W)/H);

Z = zeros(npts, nfr);
E = zeros(1, nfr);

% Hann window
%ww = hann(W); % is not used here!

% time domain version - for a 240k pt signal with W=160 H=N=80,
% took 1.14 sec
%for i = 1:nfr
%  pts = (i-1)*H + [1:W];
%  WX = W.*X(pts);
%  WY = W.*Y(pts);
%  Z(:,i) = xcorr(WX, WY, N);
%  E(i) = sqrt(sum(WX.^2)*sum(WY.^2));
%end

% faster (ahem) version, identical result
% took 0.16 sec
% 2012-07-10 Dan Ellis dpwe@ee.columbia.edu
XX = fft(frame(X,W,H),2*W);
if isempty(Y)
  YY = XX;
else
  YY = fft(frame(Y,W,H),2*W);
end
xx = ifft(XX.*conj(YY));
Z = [xx([N+1:-1:2],:);xx([1:N+1],:)];
% scaling messed up
Z = Z*W*W;
E = Z(N+1,:);
