function write_label_file(ss,ee,ll,fname,labs,type,xtra)
% write_label_file(labels, fname[, strs, type, xtra])
% write_label_file(ss, ee, ll, fname[, strs, type, xtra])
%     Write labels defined by rows <start> <end> <label> of labels
%     Or ss is start times, ee is end times, and ll is labels.
%     If strs is present and nonempty, interpret <label> or ll values as index
%     into a cell array of strings.
%     type is a type of file to write (0=3col, 1=ldc11col)
% 2012-02-29 Dan Ellis dpwe@ee.columbia.edu

mynargin = nargin;

if size(ss,2) == 3
  % passed in 3-col labels as first arg; move everything else up
  if mynargin > 4; xtra = labs; end
  if mynargin > 3; type = fname; end
  if mynargin > 2; labs = ll; end
  fname = ee;
  ll = ss(:,3);
  ee = ss(:,2);
  ss = ss(:,1);
  mynargin = mynargin + 2;
end

if mynargin < 7; xtra = []; end
if mynargin < 6; type = 0; end
if mynargin < 5; labs = []; end

if length(xtra) == 0
  % single xtra info default row
  [p,n,e] = fileparts(fname);
  xtra = {{fullfile(p,n),'0','-','-','X','findNTs','','','unk', ...
            'none',''}};
end
  
ne = length(ss);

if isstr(ll)
  % single label passed in ll is repeated for every item
  llitem = ll;
  ll = cell(ne,1);
  for i = 1:ne
    ll{i} = llitem;
  end
elseif isnumeric(ll)
  % ll is a set of values
  % use label lookup table if we have it
  % else print the actual value
  llix = ll;
  ll = cell(ne,1);
  if length(labs) > 0
    for i = 1:ne
      ll{i} = labs{llix(i)};
    end
  else
    % ll column is just real numbers, but no lookup - just use the number
    for i = 1:ne
      ll{i} = sprintf('%d',llix(i));
    end
  end
end

fp = fopen(fname,'w');

if fp < 0
  error(['Cannot open file ',fname]);
end

if type == 0 || type == 1
  if type == 0
    for i = 1:ne
      fprintf(fp,'%.3f %.3f %s\n',ss(i),ee(i),ll{i});
    end
  elseif type ==1
    % dummy 11-column line
    for i = 1:ne
      xi = min(length(xtra),i);
      xtra{xi}{3} = sprintf('%.3f',ss(i));
      xtra{xi}{4} = sprintf('%.3f',ee(i));
      xtra{xi}{5} = ll{i};
      nxc = length(xtra{xi});
      for j = 1:nxc
        fprintf(fp, '%s', xtra{xi}{j});
        if j < nxc
          fprintf(fp, '\t');
        else
          fprintf(fp, '\n');
        end
      end
    end
  end
else
  error(['Label file type ',num2str(type),' not recognized']);
end

fclose(fp);
